@@ -1,18 +1,19 @@
 use crate::annis::db::token_helper;
 use crate::annis::db::token_helper::TokenHelper;
+use crate::annis::errors::GraphAnnisError;
 use crate::annis::operator::{BinaryOperator, BinaryOperatorIndex, EstimationType};
+use crate::try_as_boxed_iter;
 use crate::{
     annis::operator::{BinaryOperatorBase, BinaryOperatorSpec},
+    errors::Result,
     graph::{GraphStorage, Match},
     model::AnnotationComponentType,
     AnnotationGraph,
 };
 use graphannis_core::{
-    annostorage::MatchGroup,
     graph::{ANNIS_NS, DEFAULT_ANNO_KEY},
     types::Component,
 };
-
 use std::any::Any;
 use std::collections::HashSet;
 use std::sync::Arc;
@@ -56,7 +57,7 @@ impl BinaryOperatorSpec for IdenticalCoverageSpec {
         v
     }
 
-    fn create_operator<'a>(&self, db: &'a AnnotationGraph) -> Option<BinaryOperator<'a>> {
+    fn create_operator<'a>(&self, db: &'a AnnotationGraph) -> Result<BinaryOperator<'a>> {
         let optional_op = IdenticalCoverage::new(db);
         optional_op.map(|op| BinaryOperator::Index(Box::new(op)))
     }
@@ -71,11 +72,19 @@ impl BinaryOperatorSpec for IdenticalCoverageSpec {
 }
 
 impl<'a> IdenticalCoverage<'a> {
-    pub fn new(db: &'a AnnotationGraph) -> Option<IdenticalCoverage<'a>> {
-        let gs_left = db.get_graphstorage(&COMPONENT_LEFT)?;
-        let gs_order = db.get_graphstorage(&COMPONENT_ORDER)?;
+    pub fn new(db: &'a AnnotationGraph) -> Result<IdenticalCoverage<'a>> {
+        let gs_left = db.get_graphstorage(&COMPONENT_LEFT).ok_or_else(|| {
+            GraphAnnisError::ImpossibleSearch(
+                "LeftToken component is missing (needed by _=_ operator)".to_string(),
+            )
+        })?;
+        let gs_order = db.get_graphstorage(&COMPONENT_ORDER).ok_or_else(|| {
+            GraphAnnisError::ImpossibleSearch(
+                "Ordering component is missing for (needed by _=_ operator)".to_string(),
+            )
+        })?;
 
-        Some(IdenticalCoverage {
+        Ok(IdenticalCoverage {
             gs_left,
             gs_order,
             tok_helper: TokenHelper::new(db)?,
@@ -90,33 +99,40 @@ impl<'a> std::fmt::Display for IdenticalCoverage<'a> {
 }
 
 impl<'a> BinaryOperatorBase for IdenticalCoverage<'a> {
-    fn filter_match(&self, lhs: &Match, rhs: &Match) -> bool {
-        let start_lhs = self.tok_helper.left_token_for(lhs.node);
-        let end_lhs = self.tok_helper.right_token_for(lhs.node);
+    fn filter_match(&self, lhs: &Match, rhs: &Match) -> Result<bool> {
+        let start_lhs = self.tok_helper.left_token_for(lhs.node)?;
+        let end_lhs = self.tok_helper.right_token_for(lhs.node)?;
 
-        let start_rhs = self.tok_helper.left_token_for(rhs.node);
-        let end_rhs = self.tok_helper.right_token_for(rhs.node);
+        let start_rhs = self.tok_helper.left_token_for(rhs.node)?;
+        let end_rhs = self.tok_helper.right_token_for(rhs.node)?;
 
-        if start_lhs.is_none() || end_lhs.is_none() || start_rhs.is_none() || end_rhs.is_none() {
-            return false;
+        if let (Some(start_lhs), Some(end_lhs), Some(start_rhs), Some(end_rhs)) =
+            (start_lhs, end_lhs, start_rhs, end_rhs)
+        {
+            let result = start_lhs == start_rhs && end_lhs == end_rhs;
+            Ok(result)
+        } else {
+            Ok(false)
         }
-
-        start_lhs.unwrap() == start_rhs.unwrap() && end_lhs.unwrap() == end_rhs.unwrap()
     }
 
     fn is_reflexive(&self) -> bool {
         false
     }
 
-    fn get_inverse_operator<'b>(&self, graph: &'b AnnotationGraph) -> Option<BinaryOperator<'b>> {
-        Some(BinaryOperator::Index(Box::new(IdenticalCoverage {
+    fn get_inverse_operator<'b>(
+        &self,
+        graph: &'b AnnotationGraph,
+    ) -> Result<Option<BinaryOperator<'b>>> {
+        let inverse = BinaryOperator::Index(Box::new(IdenticalCoverage {
             gs_left: self.gs_left.clone(),
             gs_order: self.gs_order.clone(),
             tok_helper: TokenHelper::new(graph)?,
-        })))
+        }));
+        Ok(Some(inverse))
     }
 
-    fn estimation_type(&self) -> EstimationType {
+    fn estimation_type(&self) -> Result<EstimationType> {
         if let Some(order_stats) = self.gs_order.get_statistics() {
             let num_of_token = order_stats.nodes as f64;
 
@@ -126,40 +142,49 @@ impl<'a> BinaryOperatorBase for IdenticalCoverage<'a> {
             // The probability for the same length is taken is assumed to be 1.0, histograms
             // of the distribution would help here.
 
-            EstimationType::Selectivity(1.0 / num_of_token)
+            Ok(EstimationType::Selectivity(1.0 / num_of_token))
         } else {
-            EstimationType::Selectivity(0.1)
+            Ok(EstimationType::Selectivity(0.1))
         }
     }
 }
 
 impl<'a> BinaryOperatorIndex for IdenticalCoverage<'a> {
-    fn retrieve_matches(&self, lhs: &Match) -> Box<dyn Iterator<Item = Match>> {
-        let n_left = self.tok_helper.left_token_for(lhs.node);
-        let n_right = self.tok_helper.right_token_for(lhs.node);
+    fn retrieve_matches(&self, lhs: &Match) -> Box<dyn Iterator<Item = Result<Match>>> {
+        let n_left = try_as_boxed_iter!(self.tok_helper.left_token_for(lhs.node));
+        let n_right = try_as_boxed_iter!(self.tok_helper.right_token_for(lhs.node));
 
-        let mut result = MatchGroup::new();
+        let mut result = Vec::new();
 
         if let (Some(n_left), Some(n_right)) = (n_left, n_right) {
             if n_left == n_right {
                 // covered range is exactly one token, add token itself
-                result.push(Match {
+                result.push(Ok(Match {
                     node: n_left,
                     anno_key: DEFAULT_ANNO_KEY.clone(),
-                });
+                }));
             }
 
             // find left-aligned non-token
-            let v = self.gs_left.get_ingoing_edges(n_left);
-            for c in v {
-                // check if also right-aligned
-                if let Some(c_right) = self.tok_helper.right_token_for(c) {
-                    if n_right == c_right {
-                        result.push(Match {
-                            node: c,
-                            anno_key: DEFAULT_ANNO_KEY.clone(),
-                        });
+            for c in self.gs_left.get_ingoing_edges(n_left) {
+                match c {
+                    Ok(c) => {
+                        // check if also right-aligned
+                        match self.tok_helper.right_token_for(c) {
+                            Ok(c_right) => {
+                                if let Some(c_right) = c_right {
+                                    if n_right == c_right {
+                                        result.push(Ok(Match {
+                                            node: c,
+                                            anno_key: DEFAULT_ANNO_KEY.clone(),
+                                        }));
+                                    }
+                                }
+                            }
+                            Err(e) => result.push(Err(e)),
+                        }
                     }
+                    Err(e) => result.push(Err(e.into())),
                 }
             }
         }
