@@ -1,10 +1,12 @@
 use crate::annis::db::token_helper;
 use crate::annis::db::{aql::model::AnnotationComponentType, token_helper::TokenHelper};
+use crate::annis::errors::GraphAnnisError;
 use crate::annis::operator::{BinaryOperator, BinaryOperatorSpec};
 use crate::annis::operator::{BinaryOperatorBase, BinaryOperatorIndex};
-use crate::AnnotationGraph;
-use crate::{annis::operator::EstimationType, graph::Match};
+use crate::{annis::operator::EstimationType, errors::Result, graph::Match};
+use crate::{try_as_boxed_iter, AnnotationGraph};
 use graphannis_core::{graph::DEFAULT_ANNO_KEY, types::Component};
+use itertools::Itertools;
 use std::any::Any;
 use std::collections::HashSet;
 use std::sync::Arc;
@@ -27,7 +29,7 @@ impl BinaryOperatorSpec for LeftAlignmentSpec {
         v
     }
 
-    fn create_operator<'a>(&self, db: &'a AnnotationGraph) -> Option<BinaryOperator<'a>> {
+    fn create_operator<'a>(&self, db: &'a AnnotationGraph) -> Result<BinaryOperator<'a>> {
         let optional_op = LeftAlignment::new(db);
         optional_op.map(|op| BinaryOperator::Index(Box::new(op)))
     }
@@ -42,10 +44,10 @@ impl BinaryOperatorSpec for LeftAlignmentSpec {
 }
 
 impl<'a> LeftAlignment<'a> {
-    pub fn new(graph: &'a AnnotationGraph) -> Option<LeftAlignment<'a>> {
+    pub fn new(graph: &'a AnnotationGraph) -> Result<LeftAlignment<'a>> {
         let tok_helper = TokenHelper::new(graph)?;
 
-        Some(LeftAlignment { tok_helper })
+        Ok(LeftAlignment { tok_helper })
     }
 }
 
@@ -56,14 +58,14 @@ impl<'a> std::fmt::Display for LeftAlignment<'a> {
 }
 
 impl<'a> BinaryOperatorBase for LeftAlignment<'a> {
-    fn filter_match(&self, lhs: &Match, rhs: &Match) -> bool {
+    fn filter_match(&self, lhs: &Match, rhs: &Match) -> Result<bool> {
         if let (Some(lhs_token), Some(rhs_token)) = (
-            self.tok_helper.left_token_for(lhs.node),
-            self.tok_helper.left_token_for(rhs.node),
+            self.tok_helper.left_token_for(lhs.node)?,
+            self.tok_helper.left_token_for(rhs.node)?,
         ) {
-            lhs_token == rhs_token
+            Ok(lhs_token == rhs_token)
         } else {
-            false
+            Ok(false)
         }
     }
 
@@ -71,47 +73,56 @@ impl<'a> BinaryOperatorBase for LeftAlignment<'a> {
         false
     }
 
-    fn get_inverse_operator<'b>(&self, graph: &'b AnnotationGraph) -> Option<BinaryOperator<'b>> {
+    fn get_inverse_operator<'b>(
+        &self,
+        graph: &'b AnnotationGraph,
+    ) -> Result<Option<BinaryOperator<'b>>> {
         let tok_helper = TokenHelper::new(graph)?;
 
-        Some(BinaryOperator::Index(Box::new(LeftAlignment {
+        Ok(Some(BinaryOperator::Index(Box::new(LeftAlignment {
             tok_helper,
-        })))
+        }))))
     }
 
-    fn estimation_type(&self) -> EstimationType {
+    fn estimation_type(&self) -> Result<EstimationType> {
         if let Some(stats_left) = self.tok_helper.get_gs_left_token().get_statistics() {
             let aligned_nodes_per_token: f64 = stats_left.inverse_fan_out_99_percentile as f64;
-            return EstimationType::Selectivity(
+            return Ok(EstimationType::Selectivity(
                 aligned_nodes_per_token / (stats_left.nodes as f64),
-            );
+            ));
         }
 
-        EstimationType::Selectivity(0.1)
+        Ok(EstimationType::Selectivity(0.1))
     }
 }
 
 impl<'a> BinaryOperatorIndex for LeftAlignment<'a> {
-    fn retrieve_matches(&self, lhs: &Match) -> Box<dyn Iterator<Item = Match>> {
+    fn retrieve_matches(&self, lhs: &Match) -> Box<dyn Iterator<Item = Result<Match>>> {
         let mut aligned = Vec::default();
 
-        if let Some(lhs_token) = self.tok_helper.left_token_for(lhs.node) {
-            aligned.push(Match {
+        let lhs_token = try_as_boxed_iter!(self.tok_helper.left_token_for(lhs.node));
+
+        if let Some(lhs_token) = lhs_token {
+            aligned.push(Ok(Match {
                 node: lhs_token,
                 anno_key: DEFAULT_ANNO_KEY.clone(),
-            });
+            }));
             aligned.extend(
                 self.tok_helper
                     .get_gs_left_token()
                     .get_ingoing_edges(lhs_token)
-                    .map(|n| Match {
+                    .map_ok(|n| Match {
                         node: n,
                         anno_key: DEFAULT_ANNO_KEY.clone(),
                     }),
             );
         }
 
-        Box::from(aligned.into_iter())
+        Box::from(
+            aligned
+                .into_iter()
+                .map(|m| m.map_err(GraphAnnisError::from)),
+        )
     }
 
     fn as_binary_operator(&self) -> &dyn BinaryOperatorBase {
