@@ -1,17 +1,20 @@
 use crate::annis::db::aql::operators::RangeSpec;
 use crate::annis::db::token_helper;
 use crate::annis::db::token_helper::TokenHelper;
+use crate::annis::errors::GraphAnnisError;
 use crate::annis::operator::{BinaryOperator, BinaryOperatorIndex, EstimationType};
-use crate::AnnotationGraph;
 use crate::{
     annis::operator::{BinaryOperatorBase, BinaryOperatorSpec},
+    errors::Result,
     graph::{GraphStorage, Match},
     model::{AnnotationComponent, AnnotationComponentType},
 };
+use crate::{try_as_boxed_iter, AnnotationGraph};
 use graphannis_core::graph::{ANNIS_NS, DEFAULT_ANNO_KEY, DEFAULT_NS};
+use itertools::Itertools;
 
 use std::any::Any;
-use std::collections::{HashSet, VecDeque};
+use std::collections::HashSet;
 use std::sync::Arc;
 
 #[derive(Clone, Debug, Hash, PartialEq, Eq, PartialOrd, Ord)]
@@ -66,9 +69,9 @@ impl BinaryOperatorSpec for PrecedenceSpec {
         v
     }
 
-    fn create_operator<'a>(&self, db: &'a AnnotationGraph) -> Option<BinaryOperator<'a>> {
-        let optional_op = Precedence::new(db, self.clone());
-        optional_op.map(|op| BinaryOperator::Index(Box::new(op)))
+    fn create_operator<'a>(&self, db: &'a AnnotationGraph) -> Result<BinaryOperator<'a>> {
+        let op = Precedence::new(db, self.clone())?;
+        Ok(BinaryOperator::Index(Box::new(op)))
     }
 
     fn into_any(self: Arc<Self>) -> Arc<dyn Any> {
@@ -91,7 +94,7 @@ impl std::fmt::Display for PrecedenceSpec {
 }
 
 impl<'a> Precedence<'a> {
-    pub fn new(graph: &'a AnnotationGraph, spec: PrecedenceSpec) -> Option<Precedence<'a>> {
+    pub fn new(graph: &'a AnnotationGraph, spec: PrecedenceSpec) -> Result<Precedence<'a>> {
         let ordering_layer = if spec.segmentation.is_none() {
             ANNIS_NS.to_owned()
         } else {
@@ -103,13 +106,25 @@ impl<'a> Precedence<'a> {
             spec.segmentation.clone().unwrap_or_default().into(),
         );
 
-        let gs_order = graph.get_graphstorage(&component_order)?;
-        let gs_left = graph.get_graphstorage(&COMPONENT_LEFT)?;
-        let gs_right = graph.get_graphstorage(&COMPONENT_RIGHT)?;
+        let gs_order = graph.get_graphstorage(&component_order).ok_or_else(|| {
+            GraphAnnisError::ImpossibleSearch(
+                "Ordering component missing (needed for . operator)".to_string(),
+            )
+        })?;
+        let gs_left = graph.get_graphstorage(&COMPONENT_LEFT).ok_or_else(|| {
+            GraphAnnisError::ImpossibleSearch(
+                "LeftToken component missing (needed for . operator)".to_string(),
+            )
+        })?;
+        let gs_right = graph.get_graphstorage(&COMPONENT_RIGHT).ok_or_else(|| {
+            GraphAnnisError::ImpossibleSearch(
+                "RightToken component missing (needed for . operator)".to_string(),
+            )
+        })?;
 
         let tok_helper = TokenHelper::new(graph)?;
 
-        Some(Precedence {
+        Ok(Precedence {
             gs_order,
             gs_left,
             gs_right,
@@ -126,27 +141,30 @@ impl<'a> std::fmt::Display for Precedence<'a> {
 }
 
 impl<'a> BinaryOperatorBase for Precedence<'a> {
-    fn filter_match(&self, lhs: &Match, rhs: &Match) -> bool {
+    fn filter_match(&self, lhs: &Match, rhs: &Match) -> Result<bool> {
         let start_end = if self.spec.segmentation.is_some() {
             (lhs.node, rhs.node)
         } else {
-            let start = self.tok_helper.right_token_for(lhs.node);
-            let end = self.tok_helper.left_token_for(rhs.node);
-            if start.is_none() || end.is_none() {
-                return false;
+            let start = self.tok_helper.right_token_for(lhs.node)?;
+            let end = self.tok_helper.left_token_for(rhs.node)?;
+
+            if let (Some(start), Some(end)) = (start, end) {
+                (start, end)
+            } else {
+                return Ok(false);
             }
-            (start.unwrap(), end.unwrap())
         };
 
-        self.gs_order.is_connected(
+        let result = self.gs_order.is_connected(
             start_end.0,
             start_end.1,
             self.spec.dist.min_dist(),
             self.spec.dist.max_dist(),
-        )
+        )?;
+        Ok(result)
     }
 
-    fn estimation_type(&self) -> EstimationType {
+    fn estimation_type(&self) -> Result<EstimationType> {
         if let Some(stats_order) = self.gs_order.get_statistics() {
             let max_dist = match self.spec.dist.max_dist() {
                 std::ops::Bound::Unbounded => usize::max_value(),
@@ -156,19 +174,22 @@ impl<'a> BinaryOperatorBase for Precedence<'a> {
             let max_possible_dist = std::cmp::min(max_dist, stats_order.max_depth);
             let num_of_descendants = max_possible_dist - self.spec.dist.min_dist() + 1;
 
-            return EstimationType::Selectivity(
+            return Ok(EstimationType::Selectivity(
                 (num_of_descendants as f64) / (stats_order.nodes as f64 / 2.0),
-            );
+            ));
         }
 
-        EstimationType::Selectivity(0.1)
+        Ok(EstimationType::Selectivity(0.1))
     }
 
-    fn get_inverse_operator<'b>(&self, graph: &'b AnnotationGraph) -> Option<BinaryOperator<'b>> {
+    fn get_inverse_operator<'b>(
+        &self,
+        graph: &'b AnnotationGraph,
+    ) -> Result<Option<BinaryOperator<'b>>> {
         // Check if order graph storages has the same inverse cost.
         // If not, we don't provide an inverse operator, because the plans would not account for the different costs
         if !self.gs_order.inverse_has_same_cost() {
-            return None;
+            return Ok(None);
         }
 
         let inv_precedence = InversePrecedence {
@@ -178,44 +199,50 @@ impl<'a> BinaryOperatorBase for Precedence<'a> {
             tok_helper: TokenHelper::new(graph)?,
             spec: self.spec.clone(),
         };
-        Some(BinaryOperator::Index(Box::new(inv_precedence)))
+        Ok(Some(BinaryOperator::Index(Box::new(inv_precedence))))
     }
 }
 
 impl<'a> BinaryOperatorIndex for Precedence<'a> {
-    #[allow(clippy::needless_collect)]
-    fn retrieve_matches(&self, lhs: &Match) -> Box<dyn Iterator<Item = Match>> {
+    fn retrieve_matches<'b>(&'b self, lhs: &Match) -> Box<dyn Iterator<Item = Result<Match>> + 'b> {
         let start = if self.spec.segmentation.is_some() {
             Some(lhs.node)
         } else {
-            self.tok_helper.right_token_for(lhs.node)
+            try_as_boxed_iter!(self.tok_helper.right_token_for(lhs.node))
         };
 
         if start.is_none() {
-            return Box::new(std::iter::empty::<Match>());
+            return Box::new(std::iter::empty());
         }
 
         let start = start.unwrap();
 
-        // materialize a list of all matches
-        let result: VecDeque<Match> = self
+        let connected = self
             .gs_order
             // get all token in the range
             .find_connected(start, self.spec.dist.min_dist(), self.spec.dist.max_dist())
-            .fuse()
+            .map(|t| t.map_err(GraphAnnisError::from))
+            .fuse();
+        let result = connected
             // find all left aligned nodes for this token and add it together with the token itself
-            .flat_map(move |t| {
+            .map_ok(move |t| {
                 let it_aligned = self.gs_left.get_ingoing_edges(t);
-                std::iter::once(t).chain(it_aligned)
+                std::iter::once(Ok(t)).chain(it_aligned)
             })
-            // map the result as match
-            .map(|n| Match {
-                node: n,
-                anno_key: DEFAULT_ANNO_KEY.clone(),
-            })
-            .collect();
+            .flatten_ok()
+            // Unwrap the Result<Result<_>>
+            .map(|c| match c {
+                Ok(c) => match c {
+                    Ok(n) => Ok(Match {
+                        node: n,
+                        anno_key: DEFAULT_ANNO_KEY.clone(),
+                    }),
+                    Err(e) => Err(GraphAnnisError::from(e)),
+                },
+                Err(e) => Err(e),
+            });
 
-        Box::new(result.into_iter())
+        Box::new(result)
     }
 
     fn as_binary_operator(&self) -> &dyn BinaryOperatorBase {
@@ -238,27 +265,33 @@ impl<'a> std::fmt::Display for InversePrecedence<'a> {
 }
 
 impl<'a> BinaryOperatorBase for InversePrecedence<'a> {
-    fn filter_match(&self, lhs: &Match, rhs: &Match) -> bool {
+    fn filter_match(&self, lhs: &Match, rhs: &Match) -> Result<bool> {
         let start_end = if self.spec.segmentation.is_some() {
             (lhs.node, rhs.node)
         } else {
-            let start = self.tok_helper.left_token_for(lhs.node);
-            let end = self.tok_helper.right_token_for(rhs.node);
-            if start.is_none() || end.is_none() {
-                return false;
+            let start = self.tok_helper.left_token_for(lhs.node)?;
+            let end = self.tok_helper.right_token_for(rhs.node)?;
+
+            if let (Some(start), Some(end)) = (start, end) {
+                (start, end)
+            } else {
+                return Ok(false);
             }
-            (start.unwrap(), end.unwrap())
         };
 
-        self.gs_order.is_connected(
+        let result = self.gs_order.is_connected(
             start_end.1,
             start_end.0,
             self.spec.dist.min_dist(),
             self.spec.dist.max_dist(),
-        )
+        )?;
+        Ok(result)
     }
 
-    fn get_inverse_operator<'b>(&self, graph: &'b AnnotationGraph) -> Option<BinaryOperator<'b>> {
+    fn get_inverse_operator<'b>(
+        &self,
+        graph: &'b AnnotationGraph,
+    ) -> Result<Option<BinaryOperator<'b>>> {
         let prec = Precedence {
             gs_order: self.gs_order.clone(),
             gs_left: self.gs_left.clone(),
@@ -266,10 +299,10 @@ impl<'a> BinaryOperatorBase for InversePrecedence<'a> {
             tok_helper: TokenHelper::new(graph)?,
             spec: self.spec.clone(),
         };
-        Some(BinaryOperator::Index(Box::new(prec)))
+        Ok(Some(BinaryOperator::Index(Box::new(prec))))
     }
 
-    fn estimation_type(&self) -> EstimationType {
+    fn estimation_type(&self) -> Result<EstimationType> {
         if let Some(stats_order) = self.gs_order.get_statistics() {
             let max_dist = match self.spec.dist.max_dist() {
                 std::ops::Bound::Unbounded => usize::max_value(),
@@ -279,49 +312,54 @@ impl<'a> BinaryOperatorBase for InversePrecedence<'a> {
             let max_possible_dist = std::cmp::min(max_dist, stats_order.max_depth);
             let num_of_descendants = max_possible_dist - self.spec.dist.min_dist() + 1;
 
-            return EstimationType::Selectivity(
+            return Ok(EstimationType::Selectivity(
                 (num_of_descendants as f64) / (stats_order.nodes as f64 / 2.0),
-            );
+            ));
         }
 
-        EstimationType::Selectivity(0.1)
+        Ok(EstimationType::Selectivity(0.1))
     }
 }
 
 impl<'a> BinaryOperatorIndex for InversePrecedence<'a> {
-    #[allow(clippy::needless_collect)]
-    fn retrieve_matches(&self, lhs: &Match) -> Box<dyn Iterator<Item = Match>> {
+    fn retrieve_matches<'b>(&'b self, lhs: &Match) -> Box<dyn Iterator<Item = Result<Match>> + 'b> {
         let start = if self.spec.segmentation.is_some() {
             Some(lhs.node)
         } else {
-            self.tok_helper.left_token_for(lhs.node)
+            try_as_boxed_iter!(self.tok_helper.left_token_for(lhs.node))
         };
 
         if start.is_none() {
-            return Box::new(std::iter::empty::<Match>());
+            return Box::new(std::iter::empty());
         }
 
         let start = start.unwrap();
 
-        // materialize a list of all matches
-        let result: VecDeque<Match> = self
+        let token_in_range = self
             .gs_order
             // get all token in the range
             .find_connected_inverse(start, self.spec.dist.min_dist(), self.spec.dist.max_dist())
-            .fuse()
-            // find all right aligned nodes for this token and add it together with the token itself
-            .flat_map(move |t| {
-                let it_aligned = self.gs_right.get_ingoing_edges(t);
-                std::iter::once(t).chain(it_aligned)
-            })
-            // map the result as match
-            .map(|n| Match {
-                node: n,
-                anno_key: DEFAULT_ANNO_KEY.clone(),
-            })
-            .collect();
+            .fuse();
 
-        Box::new(result.into_iter())
+        let result = token_in_range
+            .map_ok(move |t| {
+                // find all right aligned nodes for this token and add it together with the token itself
+                let it_aligned = self.gs_right.get_ingoing_edges(t);
+                std::iter::once(Ok(t)).chain(it_aligned)
+            })
+            .flatten_ok()
+            // Unwrap the Result<Result<_>>
+            .map(|c| match c {
+                Ok(c) => match c {
+                    Ok(n) => Ok(Match {
+                        node: n,
+                        anno_key: DEFAULT_ANNO_KEY.clone(),
+                    }),
+                    Err(e) => Err(GraphAnnisError::from(e)),
+                },
+                Err(e) => Err(GraphAnnisError::from(e)),
+            });
+        Box::new(result)
     }
 
     fn as_binary_operator(&self) -> &dyn BinaryOperatorBase {
