@@ -5,9 +5,11 @@ use std::path::PathBuf;
 
 use crate::annis::db::corpusstorage::get_read_or_error;
 use crate::annis::db::{aql::model::AnnotationComponentType, example_generator};
+use crate::annis::errors::GraphAnnisError;
 use crate::corpusstorage::{ImportFormat, QueryLanguage};
+use crate::errors::Result;
 use crate::update::{GraphUpdate, UpdateEvent};
-use crate::CorpusStorage;
+use crate::{AnnotationGraph, CorpusStorage};
 use graphannis_core::annostorage::{AnnotationStorage, ValueSearch};
 use graphannis_core::graph::NODE_NAME_KEY;
 use graphannis_core::types::Edge;
@@ -138,7 +140,7 @@ fn subgraph_with_segmentation() {
         let node_name = format!("root/doc1#seg{}", i);
         example_generator::create_token_node(&mut g, &node_name, t, Some("root/doc1"));
         g.add_event(UpdateEvent::AddNodeLabel {
-            node_name: node_name,
+            node_name,
             anno_ns: "default_ns".to_string(),
             anno_name: "seg".to_string(),
             anno_value: t.to_string(),
@@ -206,19 +208,37 @@ fn subgraph_with_segmentation() {
 
     let gs_cov = graph.get_graphstorage(&cov_components[0]).unwrap();
 
-    let segl0_id = graph.get_node_id_from_name("root/doc1#seg0").unwrap();
-    let seg0_out: Vec<NodeID> = gs_cov.get_outgoing_edges(segl0_id).collect();
-    assert_eq!(3, seg0_out.len());
+    let segl0_id = graph
+        .get_node_id_from_name("root/doc1#seg0")
+        .unwrap()
+        .unwrap();
+    let seg0_out: Result<Vec<_>> = gs_cov
+        .get_outgoing_edges(segl0_id)
+        .map(|e| e.map_err(GraphAnnisError::from))
+        .collect();
+    assert_eq!(3, seg0_out.unwrap().len());
 
-    let seg1_id = graph.get_node_id_from_name("root/doc1#seg1").unwrap();
-    let seg1_out: Vec<NodeID> = gs_cov.get_outgoing_edges(seg1_id).collect();
-    assert_eq!(2, seg1_out.len());
+    let seg1_id = graph
+        .get_node_id_from_name("root/doc1#seg1")
+        .unwrap()
+        .unwrap();
+    let seg1_out: Result<Vec<_>> = gs_cov
+        .get_outgoing_edges(seg1_id)
+        .map(|e| e.map_err(GraphAnnisError::from))
+        .collect();
+    assert_eq!(2, seg1_out.unwrap().len());
 
-    let seg2_id = graph.get_node_id_from_name("root/doc1#seg2").unwrap();
-    let seg2_out: Vec<NodeID> = gs_cov.get_outgoing_edges(seg2_id).collect();
-    assert_eq!(5, seg2_out.len());
+    let seg2_id = graph
+        .get_node_id_from_name("root/doc1#seg2")
+        .unwrap()
+        .unwrap();
+    let seg2_out: Result<Vec<_>> = gs_cov
+        .get_outgoing_edges(seg2_id)
+        .map(|e| e.map_err(GraphAnnisError::from))
+        .collect();
+    assert_eq!(5, seg2_out.unwrap().len());
 
-    assert_eq!(None, graph.get_node_id_from_name("root/doc1#seg3"));
+    assert_eq!(None, graph.get_node_id_from_name("root/doc1#seg3").unwrap());
 }
 
 fn compare_annos<T>(
@@ -231,14 +251,118 @@ fn compare_annos<T>(
 {
     assert_eq!(items1.len(), items2.len());
     for i in 0..items1.len() {
-        let mut annos1 = annos1.get_annotations_for_item(&items1[i]);
+        let mut annos1 = annos1.get_annotations_for_item(&items1[i]).unwrap();
         annos1.sort();
-        let mut annos2 = annos2.get_annotations_for_item(&items2[i]);
+        let mut annos2 = annos2.get_annotations_for_item(&items2[i]).unwrap();
         annos2.sort();
         assert_eq!(annos1, annos2);
     }
 }
 
+fn compare_corpora(g1: &AnnotationGraph, g2: &AnnotationGraph, rhs_remove_annis_coverage: bool) {
+    // Check all nodes and node annotations exist in both corpora
+    let nodes1: Vec<String> = g1
+        .get_node_annos()
+        .exact_anno_search(Some("annis"), "node_name", ValueSearch::Any)
+        .filter_map(|m| m.unwrap().extract_annotation(g1.get_node_annos()).unwrap())
+        .map(|a| a.val.into())
+        .sorted()
+        .collect();
+    let nodes2: Vec<String> = g2
+        .get_node_annos()
+        .exact_anno_search(Some("annis"), "node_name", ValueSearch::Any)
+        .filter_map(|m| m.unwrap().extract_annotation(g1.get_node_annos()).unwrap())
+        .map(|a| a.val.into())
+        .sorted()
+        .collect();
+    assert_eq!(&nodes1, &nodes2);
+
+    let nodes1: Vec<NodeID> = nodes1
+        .into_iter()
+        .filter_map(|n| g1.get_node_id_from_name(&n).unwrap())
+        .collect();
+    let nodes2: Vec<NodeID> = nodes2
+        .into_iter()
+        .filter_map(|n| g2.get_node_id_from_name(&n).unwrap())
+        .collect();
+    compare_annos(g1.get_node_annos(), g2.get_node_annos(), &nodes1, &nodes2);
+
+    // Check that the graphs have the same edges
+    let mut components1 = g1.get_all_components(None, None);
+    components1.sort();
+    let mut components2 = g2.get_all_components(None, None);
+    if rhs_remove_annis_coverage {
+        // Remove the special annis coverage component created during relANNIS import
+        components2.retain(|c| {
+            c.get_type() != AnnotationComponentType::Coverage
+                || !c.name.is_empty()
+                || c.layer != "annis"
+        });
+    }
+    components2.sort();
+    assert_eq!(components1, components2);
+
+    for c in components1 {
+        let gs1 = g1.get_graphstorage_as_ref(&c).unwrap();
+        let gs2 = g2.get_graphstorage_as_ref(&c).unwrap();
+
+        for i in 0..nodes1.len() {
+            let start1 = nodes1[i];
+            let start2 = nodes2[i];
+
+            // Check all connected nodes for this edge
+            let targets1: Result<Vec<String>> = gs1
+                .get_outgoing_edges(start1)
+                .filter_map_ok(|target| {
+                    g1.get_node_annos()
+                        .get_value_for_item(&target, &NODE_NAME_KEY)
+                        .unwrap()
+                })
+                .map_ok(|n| n.into())
+                .map(|n| n.map_err(GraphAnnisError::from))
+                .collect();
+            let mut targets1 = targets1.unwrap();
+            targets1.sort();
+
+            let targets2: Result<Vec<String>> = gs2
+                .get_outgoing_edges(start2)
+                .filter_map_ok(|target| {
+                    g2.get_node_annos()
+                        .get_value_for_item(&target, &NODE_NAME_KEY)
+                        .unwrap()
+                })
+                .map(|n| n.map_err(GraphAnnisError::from))
+                .map_ok(|n| n.to_string())
+                .collect();
+            let mut targets2 = targets2.unwrap();
+            targets2.sort();
+            assert_eq!(targets1, targets2);
+
+            // Check the edge annotations for each edge
+            let edges1: Vec<Edge> = targets1
+                .iter()
+                .map(|t| Edge {
+                    source: start1,
+                    target: g1.get_node_id_from_name(t).unwrap().unwrap(),
+                })
+                .collect();
+            let edges2: Vec<Edge> = targets2
+                .iter()
+                .map(|t| Edge {
+                    source: start2,
+                    target: g2.get_node_id_from_name(t).unwrap().unwrap(),
+                })
+                .collect();
+            compare_annos(
+                gs1.get_anno_storage(),
+                gs2.get_anno_storage(),
+                &edges1,
+                &edges2,
+            );
+        }
+    }
+}
+
 #[test]
 fn import_salt_sample() {
     let tmp = tempfile::tempdir().unwrap();
@@ -274,94 +398,25 @@ fn import_salt_sample() {
     let lock2 = e2.read().unwrap();
     let db2 = get_read_or_error(&lock2).unwrap();
 
-    // Check all nodes and node annotations exist in both corpora
-    let nodes1: Vec<String> = db1
-        .get_node_annos()
-        .exact_anno_search(Some("annis"), "node_name", ValueSearch::Any)
-        .filter_map(|m| m.extract_annotation(db1.get_node_annos()))
-        .map(|a| a.val.into())
-        .sorted()
-        .collect();
-    let nodes2: Vec<String> = db2
-        .get_node_annos()
-        .exact_anno_search(Some("annis"), "node_name", ValueSearch::Any)
-        .filter_map(|m| m.extract_annotation(db1.get_node_annos()))
-        .map(|a| a.val.into())
-        .sorted()
-        .collect();
-    assert_eq!(&nodes1, &nodes2);
+    compare_corpora(db1, db2, true);
+}
 
-    let nodes1: Vec<NodeID> = nodes1
-        .into_iter()
-        .filter_map(|n| db1.get_node_id_from_name(&n))
-        .collect();
-    let nodes2: Vec<NodeID> = nodes2
-        .into_iter()
-        .filter_map(|n| db2.get_node_id_from_name(&n))
-        .collect();
-    compare_annos(db1.get_node_annos(), db2.get_node_annos(), &nodes1, &nodes2);
+#[test]
+fn load_legacy_binary_corpus() {
+    let cargo_dir = PathBuf::from(env!("CARGO_MANIFEST_DIR"));
 
-    // Check that the graphs have the same edges
-    let mut components1 = db1.get_all_components(None, None);
-    components1.sort();
-    let mut components2 = db2.get_all_components(None, None);
-    // Remove the special annis coverage component created during relANNIS import
-    components2.retain(|c| {
-        c.get_type() != AnnotationComponentType::Coverage || c.name != "" || c.layer != "annis"
-    });
-    components2.sort();
-    assert_eq!(components1, components2);
+    let data_dir = cargo_dir.join("tests/data");
 
-    for c in components1 {
-        let gs1 = db1.get_graphstorage_as_ref(&c).unwrap();
-        let gs2 = db2.get_graphstorage_as_ref(&c).unwrap();
+    let cs = CorpusStorage::with_auto_cache_size(&data_dir, true).unwrap();
 
-        for i in 0..nodes1.len() {
-            let start1 = nodes1[i];
-            let start2 = nodes2[i];
+    // load both legacy corpora and compare them
+    let e1 = cs.get_fully_loaded_entry("sample-disk-based").unwrap();
+    let lock1 = e1.read().unwrap();
+    let db1 = get_read_or_error(&lock1).unwrap();
 
-            // Check all connected nodes for this edge
-            let targets1: Vec<String> = gs1
-                .get_outgoing_edges(start1)
-                .filter_map(|target| {
-                    db1.get_node_annos()
-                        .get_value_for_item(&target, &NODE_NAME_KEY)
-                })
-                .map(|n| n.into())
-                .sorted()
-                .collect();
-            let targets2: Vec<String> = gs2
-                .get_outgoing_edges(start2)
-                .filter_map(|target| {
-                    db2.get_node_annos()
-                        .get_value_for_item(&target, &NODE_NAME_KEY)
-                })
-                .map(|n| n.into())
-                .sorted()
-                .collect();
-            assert_eq!(targets1, targets2);
+    let e2 = cs.get_fully_loaded_entry("sample-memory-based").unwrap();
+    let lock2 = e2.read().unwrap();
+    let db2 = get_read_or_error(&lock2).unwrap();
 
-            // Check the edge annotations for each edge
-            let edges1: Vec<Edge> = targets1
-                .iter()
-                .map(|t| Edge {
-                    source: start1,
-                    target: db1.get_node_id_from_name(t).unwrap(),
-                })
-                .collect();
-            let edges2: Vec<Edge> = targets2
-                .iter()
-                .map(|t| Edge {
-                    source: start2,
-                    target: db2.get_node_id_from_name(t).unwrap(),
-                })
-                .collect();
-            compare_annos(
-                gs1.get_anno_storage(),
-                gs2.get_anno_storage(),
-                &edges1,
-                &edges2,
-            );
-        }
-    }
+    compare_corpora(db1, db2, false);
 }
